import * as React from "react";
import { CancelTokenSource } from "axios";
import { TableColumn } from "../TableColumn";
import { Condition, CriteriaTableContext } from "./CriteriaTableContext";
import { CriteriaTableProps } from "./CriteriaTableProps";
import { CriteriaTableControllerContext } from "../CriteriaTableController";
export interface CriteriaTableState {
    resized?: Array<{
        id: string;
        value: number;
    }>;
    sorted?: Array<{
        id: string;
        desc: boolean;
    }>;
    cancelToken?: CancelTokenSource;
    queries: Array<Condition>;
    pageSize?: number;
    pages?: number;
    page?: number;
    data: any;
    autoFetch?: boolean;
    autoFetchDelay?: number;
}
export declare enum ControlActions {
    getAutoFetchParams = "getAutoFetchParams",
    setAutoFetchState = "setAutoFetchState",
    setAutoFetchDelay = "setAutoFetchDelay",
    resetQueries = "resetQueries",
    resetData = "resetData"
}
export declare class CriteriaTable extends React.Component<CriteriaTableProps, CriteriaTableState> {
    static readonly contextTypes: {
        initData: import("prop-types").Validator<any>;
        getCurrentVisibleData: import("prop-types").Validator<any>;
        getColumn: import("prop-types").Validator<any>;
        getCurrentData: import("prop-types").Validator<any>;
        onError: import("prop-types").Validator<any>;
        saveData: import("prop-types").Validator<any>;
        getControlAction: import("prop-types").Validator<any>;
        bindControlAction: import("prop-types").Validator<any>;
        unbindControlAction: import("prop-types").Validator<any>;
        enableCaching: import("prop-types").Validator<any>;
    };
    static readonly childContextTypes: {
        setQueries: import("prop-types").Validator<any>;
        getQueries: import("prop-types").Validator<any>;
        queriesList: import("prop-types").Validator<any>;
    };
    static readonly defaultProps: {
        onDefaults?: (state: CriteriaTableState) => () => Partial<TableColumn>[];
        onFetchData?: (state: Partial<CriteriaTableState>) => Promise<Partial<CriteriaTableState>>;
        cacheKey?: string;
        previousText?: string;
        loadingText?: string;
        noDataText?: string;
        pageText?: string;
        nextText?: string;
        rowsText?: string;
        ofText?: string;
        sorted?: {
            desc: boolean;
            id: string;
        }[];
        autoFetchDelay?: number;
        autoFetch?: boolean;
        pageSize?: number;
        pages?: number;
        page?: number;
        data?: any;
    };
    static readonly propTypes: {
        onDefaults: import("prop-types").Validator<any>;
        onFetchData: import("prop-types").Validator<any>;
        cacheKey: import("prop-types").Validator<any>;
        previousText?: import("prop-types").Validator<any>;
        loadingText?: import("prop-types").Validator<any>;
        noDataText?: import("prop-types").Validator<any>;
        pageText?: import("prop-types").Validator<any>;
        nextText?: import("prop-types").Validator<any>;
        rowsText?: import("prop-types").Validator<any>;
        ofText?: import("prop-types").Validator<any>;
        sorted?: import("prop-types").Validator<any>;
        autoFetchDelay?: import("prop-types").Validator<any>;
        autoFetch?: import("prop-types").Validator<any>;
        pageSize?: import("prop-types").Validator<any>;
        pages?: import("prop-types").Validator<any>;
        page?: import("prop-types").Validator<any>;
        data?: import("prop-types").Validator<any>;
    };
    static actionDelay: number;
    readonly context: CriteriaTableControllerContext;
    state: CriteriaTableState;
    fetchControlTimeoutId: any;
    autoFetchTimeoutId: any;
    readonly bindMap: ({
        actionName: ControlActions;
        action: (autoFetchDelay: number) => void;
    } | {
        actionName: ControlActions;
        action: (autoFetch: boolean) => void;
    })[];
    getChildContext(): CriteriaTableContext;
    componentDidUpdate(prevProps: any, prevState: CriteriaTableState): void;
    componentWillMount(): void;
    componentWillUnmount(): void;
    render(): JSX.Element;
    protected startAutoFetch: () => void;
    protected fetchDataControl: () => void;
    protected setAutoFetchDelay: (autoFetchDelay: number) => void;
    protected setAutoFetchState: (autoFetch: boolean) => void;
    protected getAutoFetchParams: () => {
        delay: number;
        enabled: boolean;
    };
    protected handlePageSizeChange: (pageSize: number) => void;
    protected handleSortedChange: (sorted: {
        id: string;
        desc: boolean;
    }[]) => void;
    protected handleResized: (resized: {
        id: string;
        value: number;
    }[]) => void;
    protected handlePageChange: (page: number) => void;
    protected handleGetQueries: () => [import("./CriteriaTableContext").ConditionType, string, string | number | boolean][];
    protected handleFetchData: () => Promise<void>;
    protected handleResetData: () => void;
    protected handleResetQueries: () => void;
    protected handleSetQueries: (conditionQueries: [import("./CriteriaTableContext").ConditionType, string, string | number | boolean][]) => void;
    protected saveData: () => void;
    protected readonly cachedState: CriteriaTableState;
    protected readonly labels: {};
    protected readonly controlledStateCallbacks: {
        onPageSizeChange: (pageSize: number) => void;
        onSortedChange: (sorted: {
            id: string;
            desc: boolean;
        }[]) => void;
        onPageChange: (page: number) => void;
        onResizedChange: (resized: {
            id: string;
            value: number;
        }[]) => void;
        onFetchData: () => void;
    };
    protected readonly controlledStateOverrides: {
        pageSize: number;
        resized: {
            id: string;
            value: number;
        }[];
        data: any;
        sorted: {
            id: string;
            desc: boolean;
        }[];
        pages: number;
        page: number;
    };
}
